package org.jtheque.core.managers.persistence;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jtheque.core.managers.persistence.context.TemporaryContext;

/**
 * Represents a persisted object of JTheque.
 *
 * @author Baptiste Wicht
 */
public abstract class Entity implements Comparable<Entity> {
    private int id;

    private final TemporaryContext temporaryContext;

    public static final int HASHCODEPRIME = 31;

    /**
     * Construct a new Entity.
     */
    public Entity() {
        super();

        temporaryContext = new TemporaryContext();
    }

    /**
     * Return the temporary context of the data.
     *
     * @return The temporary context of the data.
     */
    public TemporaryContext getTemporaryContext() {
        return temporaryContext;
    }
    
    /**
     * Return the id of this Entity.
     *
     * @return The id
     */
    public final int getId() {
        return id;
    }

    /**
     * Set the id of this Entity.
     *
     * @param id The new id to set
     */
    public final void setId(int id) {
        this.id = id;
    }

    /**
     * Return an affichable text of the Entity. This is not the same as toString(). This method
     * provide an text of display on the application.
     *
     * @return A string representation of the Entity.
     */
    public abstract String getAffichableText();

    @Override
    public final int compareTo(Entity object) {
        return getAffichableText().compareTo(object.getAffichableText());
    }

    @Override
    public abstract boolean equals(Object object);

    @Override
    public abstract int hashCode();

    /**
     * Indicate if the entity is saved or not.
     *
     * @return true if the entity is saved, else false.
     */
    public final boolean isSaved() {
        return id != 0;
    }

    /**
     * Save the state of the data object to a memento. By default this method isn't supported by Entity.
     */
    public void saveToMemento() {
        throw new UnsupportedOperationException();
    }

    /**
     * Restore the state of the data object from the memento. By default this method isn't supported by Entity.
     */
    public void restoreMemento() {
        throw new UnsupportedOperationException();
    }
}