package org.jtheque.core.managers.properties;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jtheque.core.managers.ManagerException;
import org.jtheque.core.managers.Managers;
import org.jtheque.core.managers.log.ILoggingManager;
import org.jtheque.utils.collections.CollectionUtils;
import org.jtheque.utils.Constants;

import java.beans.BeanInfo;
import java.beans.IntrospectionException;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * A properties manager implementation.
 *
 * @author Baptiste Wicht
 */
public final class PropertiesManager implements IPropertiesManager {
    private final Map<Class<?>, List<Property>> properties = new HashMap<Class<?>, List<Property>>(25);
    
    private boolean enabled = true;

    @Override
    public void preInit() {
        //Nothing to do
    }

    @Override
    public void init() throws ManagerException {
        //Nothing to init.
    }

    @Override
    public void close() throws ManagerException {
        //Nothing to init
    }

    @Override
    public boolean isEnabled() {
        return enabled;
    }

    @Override
    public void setEnabled(boolean enabled) {
        this.enabled = enabled;
    }

    @Override
    public List<Property> getProperties(Object bean) throws PropertyException {
        Class<?> beanClass = bean.getClass();

        if (!properties.containsKey(beanClass)) {
            try {
                BeanInfo info = Introspector.getBeanInfo(beanClass);

                PropertyDescriptor[] descriptors = info.getPropertyDescriptors();

                List<Property> list = new ArrayList<Property>(descriptors.length - 1);

                for (PropertyDescriptor desc : descriptors) {
                    if (!"class".equals(desc.getName()) && desc.getReadMethod() != null && desc.getWriteMethod() != null) {
                        list.add(convertToProperty(desc));
                    }
                }

                properties.put(beanClass, list);
            } catch (IntrospectionException e) {
                throw new PropertyException("Erreur lors de la rcupration des proprits", e);
            }
        }

        return properties.get(beanClass);
    }

    /**
     * Convert the property descriptor to a Property. 
     * 
     * @param desc The desc The property descriptor. 
     * 
     * @return The Property. 
     */
    private static Property convertToProperty(PropertyDescriptor desc) {
        Property property = new Property();

        property.setName(desc.getName());
        property.setGetter(desc.getReadMethod());
        property.setSetter(desc.getWriteMethod());
        
        return property;
    }

    @Override
    public Object getProperty(Object bean, Property property) {
        Method m = property.getGetter();

        Object o = null;

        try {
            o = m.invoke(bean);
        } catch (IllegalAccessException e) {
            Managers.getManager(ILoggingManager.class).getLogger(bean.getClass()).exception(e);
        } catch (InvocationTargetException e) {
            Managers.getManager(ILoggingManager.class).getLogger(bean.getClass()).exception(e);
        }

        return o;
    }

    @Override
    public Object getProperty(Object bean, String property) {
        putInCacheIfNecessary(bean);

        for (Property p : properties.get(bean.getClass())) {
            if (p.getName().equals(property)) {
                return getProperty(bean, p);
            }
        }

        return null;
    }

    private void putInCacheIfNecessary(Object bean) {
        if(properties.get(bean.getClass()) == null){
            try {
                properties.put(bean.getClass(), getProperties(bean));
            } catch (PropertyException e) {
                Managers.getManager(ILoggingManager.class).getLogger(bean.getClass()).exception(e);
            }
        }
    }

    @Override
    public <T> T createMemento(T bean) {
        T instance = null;

        try {
            instance = (T) bean.getClass().newInstance();

            for (Property property : getProperties(bean)) {
                Object value = property.getGetter().invoke(bean);

                setValue(instance, property, value);
            }
        } catch (Exception e) {
            Managers.getManager(ILoggingManager.class).getLogger(bean.getClass()).exception(e);
        }

        return instance;
    }

    @Override
    public void restoreMemento(Object bean, Object memento) {
        try {
            for (Property property : getProperties(bean)) {
                Object value = getProperty(memento, property);

                setValue(bean, property, value);
            }
        } catch (PropertyException e) {
            Managers.getManager(ILoggingManager.class).getLogger(bean.getClass()).exception(e);
        } catch (InvocationTargetException e) {
            Managers.getManager(ILoggingManager.class).getLogger(bean.getClass()).exception(e);
        } catch (IllegalAccessException e) {
            Managers.getManager(ILoggingManager.class).getLogger(bean.getClass()).exception(e);
        }
    }

    /**
     * Set the value to the bean on the bean. 
     * 
     * @param bean The bean to edit. 
     * @param property The property to set. 
     * @param value The value to set to the property. 
     * 
     * @throws IllegalAccessException If the property cannot be accessed. 
     * @throws InvocationTargetException If there is a problem during the setting process. 
     */
    private static void setValue(Object bean, Property property, Object value) throws IllegalAccessException, InvocationTargetException {
        if (value != null && List.class.isAssignableFrom(value.getClass())) {
            property.getSetter().invoke(bean, CollectionUtils.copyOf((Collection<?>)value));
        } else {
            property.getSetter().invoke(bean, value);
        }
    }

    @Override
    public String toString(Object bean) {
        StringBuilder builder = new StringBuilder(bean.getClass().getSimpleName());

        builder.append('{');

        try {
            for (Property property : getProperties(bean)) {
                builder.append(property.getName()).append('=');
                builder.append(getProperty(bean, property));
                builder.append(", ");
            }

            builder.delete(builder.lastIndexOf(", "), builder.lastIndexOf(", ") + 2);
        } catch (PropertyException e) {
            Managers.getManager(ILoggingManager.class).getLogger(bean.getClass()).exception(e);
        }

        builder.append('}');

        return builder.toString();
    }

    @Override
    public int hashCode(Object bean) {
        int result = Constants.HASH_CODE_START;

        try {
            for (Property property : getProperties(bean)) {
                Object value = getProperty(bean, property);

                result = computeValue(result, value);
            }
        } catch (PropertyException e) {
            Managers.getManager(ILoggingManager.class).getLogger(bean.getClass()).exception(e);
        }

        return result;
    }

    /**
     * Compute the value with the current result. 
     * 
     * @param result The current hash code result. 
     * @param value The value to compute to the result. 
     * 
     * @return The result computed with the value. 
     */
    private static int computeValue(int result, Object value) {
        int hash;

        if (value instanceof Double) {
            Long temp = Double.doubleToLongBits((Double) value);
            hash = Constants.HASH_CODE_PRIME * result + (int) (temp ^ temp >>> 32);
        } else if (value instanceof Long) {
            Long temp = (Long)value;
            hash = Constants.HASH_CODE_PRIME * result + (int) (temp ^ temp >>> 32);
        } else if (value instanceof Boolean) {
            hash = Constants.HASH_CODE_PRIME * result + ((Boolean) value ? 0 : 1);
        } else if (value instanceof Float) {
            hash = Constants.HASH_CODE_PRIME * result + Float.floatToIntBits((Float) value);
        } else if (value instanceof Number) {
            hash = Constants.HASH_CODE_PRIME * result + ((Number) value).intValue();
        } else {
            hash = Constants.HASH_CODE_PRIME * result + (value == null ? 0 : value.hashCode());
        }

        return hash;
    }
}