package org.jtheque.core.managers.skin;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import com.l2fprod.gui.plaf.skin.SkinLookAndFeel;
import org.jtheque.core.managers.Managers;
import org.jtheque.core.managers.AbstractManager;

import javax.swing.UIManager;
import javax.swing.UIManager.LookAndFeelInfo;
import java.io.File;
import java.io.FileFilter;
import java.util.List;
import java.util.ArrayList;

/**
 * This class manages the look and feel of the application.
 *
 * @author Baptiste Wicht
 */
public final class SkinManager extends AbstractManager implements ISkinManager {
    private final List<Skin> skins;
    private Skin currentSkin;

    private static final int NUMBER_OF_SKINS = 30;

    /**
     * Construct a new SkinManager. 
     */
    public SkinManager() {
        super();

        skins = new ArrayList<Skin>(NUMBER_OF_SKINS);
    }

    @Override
    public void preInit(){
        //Nothing to do
    }

    @Override
    public void init() {
        loadSkins();

        setSkin(Managers.getCore().getConfiguration().getPreferedLookAndFeel());
    }

    @Override
    public List<Skin> getSkins() {
        return skins;
    }

    /**
     * Load the look and feels, the system look and feels and all the themepack look and feels.
     */
    private void loadSkins() {
        UIManager.put("swing.boldMetal", Boolean.FALSE);

        loadInstalledSkins();
        loadThemePacks();

        currentSkin = skins.get(0);
    }

    /**
     * Load the installed skins. 
     */
    private void loadInstalledSkins() {
        LookAndFeelInfo[] installedLaf = UIManager.getInstalledLookAndFeels();

        for (LookAndFeelInfo laf : installedLaf) {
            skins.add(new SystemSkin(laf));
        }
    }

    /**
     * Load the theme packs skins. 
     */
    private void loadThemePacks() {
        File[] files = Managers.getCore().getFolders().getSkinsFolder().listFiles(new ThemePackFileFilter());

        for (File f : files) {
            skins.add(new ThemePackSkin(f));
        }
    }

    @Override
    public void setSkin(Skin skin) {
        if (skin.isThemePack()) {
            applyThemePack(skin);
        }

        applyLookAndFeel(skin);
    }

    /**
     * Apply the theme pack. 
     * 
     * @param skin The theme pack skin to apply. 
     */
    private void applyThemePack(Skin skin) {
        try {
            com.l2fprod.gui.plaf.skin.Skin newSkin = SkinLookAndFeel.loadThemePack(skin.getFile().getAbsolutePath());

            SkinLookAndFeel.setSkin(newSkin);
        } catch (Exception e) {
            getLogger().exception(e);
        }
    }

    /**
     * Apply the look and feel. 
     * 
     * @param skin The skin to apply. 
     */
    private void applyLookAndFeel(Skin skin) {
        try {
            UIManager.setLookAndFeel(skin.getClassName());

            currentSkin = skin;

            fireCurrentSkinChanged();
        } catch (Exception e) {
            getLogger().exception(e);
        }
    }

    /**
     * Apply the look and feel to the application, search the look and feel with the name in the list of available
     * look and feels.
     *
     * @param name The name of the skin
     */
    private void setSkin(String name) {
        if (name != null && name.length() > 0) {
            Skin skin = getSkin(name);

            if (skin == null) {
                getLogger().message("Le skin " + name + " n'a pas t trouv, un autre skin a t appliqu");
                setSkin(skins.get(0));
            } else {
                setSkin(getSkin(name));
            }
        }
    }

    /**
     * Return the look and feel with a specific name.
     *
     * @param name The name of the look and feel we search
     * @return The look and feel if we find it or <code>null</code>
     */
    private Skin getSkin(String name) {
        Skin skin = null;

        for (Skin s : skins) {
            if (s.getName().equals(name)) {
                skin = s;
                break;
            }
        }

        return skin;
    }

    @Override
    public Skin getCurrentSkin() {
        return currentSkin;
    }

    /**
     * This class is a filter to get only the themepacks on a directory.
     *
     * @author Baptiste Wicht
     */
    private static class ThemePackFileFilter implements FileFilter {
        @Override
        public boolean accept(File file) {
            return file.getName().endsWith("themepack.zip") && !file.isDirectory();
        }
    }

    @Override
    public void close() {
        //Nothing to close
    }

    @Override
    public void addSkinListener(SkinListener listener){
        getListeners().add(SkinListener.class,  listener);
    }

    @Override
    public void removeSkinListener(SkinListener listener){
        getListeners().remove(SkinListener.class,  listener);
    }

    /**
     * Fire an event to say that the current skin has changed. 
     */
    private static void fireCurrentSkinChanged() {
        SkinListener[] listeners = getListeners().getListeners(SkinListener.class);

        for(SkinListener l : listeners){
            l.skinChanged();
        }
    }
}