package org.jtheque.core.managers.update.versions;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jdom.Element;
import org.jtheque.core.managers.Managers;
import org.jtheque.core.managers.core.ICore;
import org.jtheque.core.managers.module.beans.ModuleContainer;
import org.jtheque.core.managers.log.ILoggingManager;
import org.jtheque.core.managers.patch.OnlinePatch;
import org.jtheque.core.managers.update.actions.DeleteAction;
import org.jtheque.core.managers.update.actions.DownloadAction;
import org.jtheque.core.managers.update.actions.MoveAction;
import org.jtheque.core.managers.update.actions.UpdateAction;
import org.jtheque.core.managers.update.actions.AbstractUpdateAction;
import org.jtheque.core.managers.update.Updatable;
import org.jtheque.utils.StringUtils;
import org.jtheque.utils.io.FileUtils;
import org.jtheque.utils.bean.Version;
import org.jtheque.core.utils.file.XMLException;
import org.jtheque.core.utils.file.XMLReader;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Collection;

/**
 * A reader for the versions file.
 *
 * @author Baptiste Wicht
 */
public final class VersionsFileReader {
    private final XMLReader reader = new XMLReader();
    private VersionsFile versionsFile;

    /**
     * Read a versions file and return it.
     *
     * @param object The URL of the file.
     *
     * @return The versions file.
     */
    public VersionsFile read(Object object) {
        if(isModule(object)){
            ModuleContainer module = (ModuleContainer)object;

            if(module.hasModuleDefinition()){
                return read(module.getModuleDefinition().updateURL());
            }
        } else if(isUpdatable(object)){
            Updatable updatable = (Updatable)object;

            return read(updatable.getVersionsFileURL());
        } else if(isCore(object)){
            return read(Managers.getCore().getFiles().getVersionsFileURL());
        }

        return null;
    }

    /**
     * Indicate if the object is a module or not. 
     * 
     * @param object The object to test. 
     * 
     * @return true if the object is a module else false. 
     */
    private static boolean isModule(Object object) {
        return object instanceof ModuleContainer;
    }

    /**
     * Indicate if the object is an updatable or not. 
     * 
     * @param object The object to test. 
     * 
     * @return true if the object is an updatable else false. 
     */
    private static boolean isUpdatable(Object object) {
        return object instanceof Updatable;
    }

    /**
     * Indicate if the object is the core or not. 
     * 
     * @param object The object to test. 
     * 
     * @return true if the object is the core else false. 
     */
    private static boolean isCore(Object object) {
        return object instanceof ICore;
    }

    /**
     * Read the VersionsFile at the url. 
     * 
     * @param strUrl the url string value. 
     * 
     * @return the readed versions file. 
     */
    private VersionsFile read(String strUrl){
        try {
            return readVersionsFile(new URL(strUrl));
        } catch (MalformedURLException e) {
            Managers.getManager(ILoggingManager.class).getLogger(getClass()).error("Unable to read versions file", e);
        }

        return null;
    }

    /**
     * Read a versions file and return it.
     *
     * @param url The URL of the file.
     *
     * @return The versions file.
     */
    private VersionsFile readVersionsFile(URL url) {
        versionsFile = new VersionsFile();
        versionsFile.setVersions(new ArrayList<OnlineVersion>(10));

        try {
            reader.openURL(url);

            readInstallVersion();

            for (Object currentNode : reader.getNodes("jt-version", reader.getRootElement())) {
                readOnlineVersion(currentNode);
            }

            Collections.sort(versionsFile.getVersions());
        } catch (XMLException e) {
            Managers.getManager(ILoggingManager.class).getLogger(getClass()).error("Unable to read versions file", e);
        } finally {
            FileUtils.close(reader);
        }

        return versionsFile;
    }

    /**
     * Read the install version. 
     * 
     * @throws XMLException If an error occurs during the xml reading. 
     */
    private void readInstallVersion() throws XMLException {
        InstallVersion installVersion = new InstallVersion();

        Object installVersionNode = reader.getNode("install", reader.getRootElement());

        readOnlineVersion(installVersionNode, installVersion);

        installVersion.setJarFile(reader.readString("jar-file", installVersionNode));
        installVersion.setTitle(reader.readString("title", installVersionNode));

        versionsFile.setInstallVersion(installVersion);
    }

    /**
     * Read the patchs of the current node.
     *
     * @param currentNode The current node.
     * @param version The version to read the patchs for.
     *
     * @throws XMLException If an errors occurs during the reading process.
     */
    private void readPatchs(Object currentNode, OnlineVersion version) throws XMLException {
        List<Element> nodes = reader.getNodes("patchs/patch", currentNode);

        Collection<OnlinePatch> patchs = new ArrayList<OnlinePatch>(nodes.size());

        for (Object node : nodes) {
            OnlinePatch patch = new OnlinePatch();
            patch.setClassName(reader.readString("@class", node));

            patchs.add(patch);
        }

        version.setPatchs(patchs);
    }

    /**
     * Fill an online version.
     *
     * @param currentNode   The current node.
     * @throws XMLException If an error occurs during the reading process.
     */
    private void readOnlineVersion(Object currentNode) throws XMLException {
        OnlineVersion onlineVersion = new OnlineVersion();

        readOnlineVersion(currentNode, onlineVersion);

        versionsFile.getVersions().add(onlineVersion);
    }

    /**
     * Read the online version. 
     * 
     * @param currentNode The node to read from. 
     * @param onlineVersion The online version to fill. 
     * 
     * @throws XMLException If an error occurs during the reading process.
     */
    private void readOnlineVersion(Object currentNode, OnlineVersion onlineVersion) throws XMLException {
        readPatchs(currentNode, onlineVersion);
        readVersionNumber(currentNode, onlineVersion);
        readCoreVersion(currentNode, onlineVersion);
        readActions(currentNode, onlineVersion);
    }

    /**
     * Read the version number. 
     * 
     * @param currentNode The node to read from. 
     * @param onlineVersion The online version to fill. 
     * 
     * @throws XMLException If an error occurs during the reading process.
     */
    private void readVersionNumber(Object currentNode, OnlineVersion onlineVersion) throws XMLException {
        onlineVersion.setVersion(new Version(reader.readString("nom", currentNode)));
    }

    /**
     * Read the actions of the online version. 
     * 
     * @param currentNode The node to read from. 
     * @param onlineVersion The online version to fill. 
     * 
     * @throws XMLException If an error occurs during the reading process.
     */
    private void readActions(Object currentNode, OnlineVersion onlineVersion) throws XMLException {
        List<Element> nodes = reader.getNodes("actions/*", currentNode);

        onlineVersion.setActions(new ArrayList<UpdateAction>(nodes.size()));

        for (Object node : nodes) {
            String name = ((Element) node).getName();

            if ("add".equals(name)) {
                AbstractUpdateAction action = readDownloadAction(node);

                onlineVersion.getActions().add(action);
            } else if ("delete".equals(name)) {
                AbstractUpdateAction action = readDeleteAction(node);

                onlineVersion.getActions().add(action);
            } else if ("move".equals(name)) {
                AbstractUpdateAction action = readMoveAction(node);

                onlineVersion.getActions().add(action);
            }
        }
    }

    /**
     * Read the download action. 
     * 
     * @param node The node to read from. 
     * 
     * @return the readed DownloadAction. 
     * 
     * @throws XMLException If an error occurs during the reading process.
     */
    private AbstractUpdateAction readDownloadAction(Object node) throws XMLException {
        DownloadAction action = new DownloadAction();
        
        action.setFolder(reader.readString("folder", node));
        action.setFile(reader.readString("file", node));
        action.setUrl(reader.readString("url", node));

        return action;
    }

    /**
     * Read a delete action. 
     * 
     * @param node The node to read from. 
     * 
     * @return The delete action. 
     * 
     * @throws XMLException If an error occurs during the reading process.
     */
    private AbstractUpdateAction readDeleteAction(Object node) throws XMLException {
        AbstractUpdateAction action = new DeleteAction();

        action.setFolder(reader.readString("folder", node));
        action.setFile(reader.readString("file", node));

        return action;
    }

    /**
     * Read a move action. 
     * 
     * @param node The node to read from. 
     * 
     * @return The move action. 
     * 
     * @throws XMLException If an error occurs during the reading process.
     */
    private AbstractUpdateAction readMoveAction(Object node) throws XMLException {
        MoveAction action = new MoveAction();

        action.setSourceFile(reader.readString("src-file", node));
        action.setSourceFolder(reader.readString("src-folder", node));
        action.setFile(reader.readString("dest-file", node));
        action.setFolder(reader.readString("dest-folder", node));

        return action;
    }

    /**
     * Read the needed version of the core. 
     * 
     * @param currentNode The node to read from. 
     * @param onlineVersion The online version to fill. 
     * 
     * @throws XMLException If an error occurs during the reading process.
     */
    private void readCoreVersion(Object currentNode, OnlineVersion onlineVersion) throws XMLException {
        if (StringUtils.isEmpty(reader.readString("core", currentNode))) {
            onlineVersion.setCoreVersion(Managers.getCore().getCoreCurrentVersion());
        } else {
            onlineVersion.setCoreVersion(new Version(reader.readString("core", currentNode)));
        }
    }
}