package org.jtheque.core.managers.view.impl.components.filthy.java2d;

import org.jtheque.core.managers.module.beans.ModuleContainer;
import org.jtheque.core.managers.module.IModuleManager;
import org.jtheque.core.managers.module.annotations.PrimaryModule;
import org.jtheque.core.managers.Managers;
import org.jtheque.core.managers.language.ILanguageManager;
import org.jtheque.core.managers.resource.IResourceManager;
import org.jtheque.core.managers.view.IViewManager;
import org.jtheque.core.managers.view.able.IPrimaryListView;
import org.jtheque.utils.ui.ImageUtils;
import org.jtheque.utils.ui.PaintUtils;
import org.jtheque.core.utils.ui.AnimationUtils;

import org.jdesktop.animation.timing.Animator;
import org.jdesktop.animation.timing.TimingTargetAdapter;
import org.jdesktop.animation.timing.triggers.TimingTrigger;
import org.jdesktop.animation.timing.triggers.TimingTriggerEvent;

import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.Graphics2D;
import java.awt.Color;
import java.awt.Font;
import java.awt.Cursor;
import java.awt.Point;
import java.awt.Component;
import java.awt.event.MouseEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseMotionAdapter;
import java.awt.geom.Path2D;
import java.awt.geom.GeneralPath;
import java.awt.image.BufferedImage;
import java.util.List;
import java.util.Map;
import java.util.ArrayList;
import java.util.HashMap;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * A view to display the list of primary modules. 
 * 
 * @author Baptiste Wicht
 */
public final class PrimaryListPane extends ZoomAlphaCapableComponent implements IPrimaryListView {
    private final List<ModuleContainer> modules;
    private final Map<ModuleContainer, Rectangle> moduleRectangles;

    private Shape mouseShape;

    private final BufferedImage[] images;

    private int grow = -1;

    private static final float FACTOR = 1.5f;

    private Rectangle rectangle;
    private BufferedImage image;

    /**
     * Construct a new PrimaryListPane.  
     */
    public PrimaryListPane() {
        super();

        modules = new ArrayList<ModuleContainer>(5);
        modules.addAll(Managers.getManager(IModuleManager.class).getPrimaryModules());

        images = new BufferedImage[modules.size()];

        moduleRectangles = new HashMap<ModuleContainer, Rectangle>(modules.size());

        init();
    }

    /**
     * Init the view. 
     */
    private void init(){
        int currentY = 75;

        for(int i = 0; i < modules.size(); i++){
            generateImage(i);

            computeModuleRectangle(currentY, i);

            currentY = currentY + images[i].getHeight() + 10;
        }

        computePath();

        addMouseListener(new MouseController());
        addMouseMotionListener(new MouseMotionController());
    }

    /**
     * Compute the module rectangles with a new module rectangle one. 
     * 
     * @param currentY The current y. 
     * @param i The index of the module. 
     */
    private void computeModuleRectangle(int currentY, int i) {
        Rectangle moduleRectangle = new Rectangle(
                25, currentY,
                images[i].getWidth(), images[i].getHeight());

        moduleRectangles.put(modules.get(i), moduleRectangle);
    }

    /**
     * Generate the image of the specified module index. 
     * 
     * @param i The module index. 
     */
    private void generateImage(int i) {
        PrimaryModule primary = modules.get(i).getClass().getAnnotation(PrimaryModule.class);

        String name = modules.get(i).getInfos().name();
        String description = modules.get(i).getInfos().description();

        BufferedImage primaryModuleImage = Managers.getManager(IResourceManager.class).getImage(primary.icon());

        images[i] = ImageUtils.createCompatibleImage(
                135 + primaryModuleImage.getGraphics().getFontMetrics(primaryModuleImage.getGraphics().getFont().deriveFont(20f)).stringWidth(name) +
                        primaryModuleImage.getWidth(),
                (int) (primaryModuleImage.getHeight() * 1.5),
                BufferedImage.TYPE_INT_ARGB);

        Graphics2D g = (Graphics2D) images[i].getGraphics();

        PaintUtils.initHints(g);

        paintReflection(g, primaryModuleImage);

        PaintUtils.drawString(g, name, 135, 20, g.getFont().deriveFont(20f));
        PaintUtils.drawString(g, description, 135, 50, g.getFont().deriveFont(16f).deriveFont(Font.PLAIN));

        g.dispose();
    }

    /**
     * Paint the reflection. 
     * 
     * @param g The graphics to use. 
     * @param image The image to make the reflection from. 
     */
    private static void paintReflection(Graphics g, BufferedImage image) {
        BufferedImage reflected = ImageUtils.createReflection(image);

        int x = 0;

        if (reflected.getWidth() < 125) {
            x = (125 - reflected.getWidth()) / 2;
        }

        g.drawImage(reflected, x, 0, null);
    }

    /**
     * Compute the Path2D for all the modules rectangles.  
     */
    private void computePath() {
        Path2D path = new GeneralPath();

        for (Rectangle rect : moduleRectangles.values()) {
            path.append(rect, false);
        }

        mouseShape = path;
    }

    @Override
    public void paint(Graphics graphics) {
        Graphics2D g2 = (Graphics2D)graphics;

        PaintUtils.initHints(g2);

        paintTitle(g2);
        paintModules(g2);
        paintSpringEffect(g2);
    }

    /**
     * Paint the modules to the view. 
     * 
     * @param g2 The Graphics2D element to paint to. 
     */
    private void paintModules(Graphics2D g2) {
        int currentY = 75;

        for(int i = 0; i < modules.size(); i++){
            BufferedImage moduleImage = images[i];

            double factor = i == grow && getZoom() < 0.0f ? 1.1 : 1.0;

            PaintUtils.drawAlphaImage(g2, moduleImage, 25, currentY,
                    (int)(moduleImage.getWidth() * factor), (int)(moduleImage.getHeight() * factor), getAlpha());

            currentY = currentY + (int) (moduleImage.getHeight() * factor) + 10;
        }
    }

    /**
     * Paint the title to the view. 
     * 
     * @param g2 The Graphics2D element to paint to. 
     */
    private void paintTitle(Graphics g2) {
        Color color = new Color(255, 255, 255, (int)(getAlpha() * 255));

        PaintUtils.drawString(g2,
                Managers.getManager(ILanguageManager.class).getMessage("modules.primary.title"),
                25, 40, g2.getFont().deriveFont(24f).deriveFont(Font.BOLD), color);

        PaintUtils.fillRect(g2, 25, 65, getWidth() - 50, 3, color);
    }

    /**
     * Paint the spring effet to the view. 
     * 
     * @param g2 The Graphics2D element to paint to. 
     */
    private void paintSpringEffect(Graphics g2) {
        if(getZoom() >= 0.0f){
            int width = (int)( image.getWidth() + image.getWidth() * FACTOR * getZoom());
            int height = (int)( image.getHeight() + image.getHeight() * FACTOR * getZoom());

            int x = rectangle.x  + (rectangle.width - width) / 2;
            int y = rectangle.y  + (rectangle.height - height) / 2;

            Graphics2D g = (Graphics2D)g2.create();

            PaintUtils.initHints(g);

            PaintUtils.drawAlphaImage(g, image, x, y, width, height, 1.0f - getZoom());

            g.dispose();
        }
    }

    /**
     * Test if the event occured in the mouse shape. 
     * 
     * @param event The event to test. 
     * 
     * @return true if the event is in the mouse shape. 
     */
    private boolean eventInMouseShape(MouseEvent event) {
        return mouseShape.contains(event.getPoint());
    }

    /**
     * Return the module at the specified point. 
     * 
     * @param point The point to get the module for. 
     * 
     * @return The module container at the specified point. 
     */
    private ModuleContainer getModule(Point point){
        for (Map.Entry<ModuleContainer, Rectangle> entry : moduleRectangles.entrySet()) {
            if (entry.getValue().contains(point)) {
                return entry.getKey();
            }
        }

        return null;
    }

    @Override
    public void appear() {
        AnimationUtils.startFadeIn(this);
    }

    @Override
    public Component getImpl() {
        return this;
    }

    /**
     * The mouse controller of the view. 
     * 
     * @author Baptiste Wicht
     */
    private final class MouseController extends MouseAdapter {
        @Override
        public void mouseClicked(MouseEvent event) {
            if (eventInMouseShape(event)) {
                launchNextPhase(event);
                closeView(event);
            }
        }

        /**
         * Close the view. 
         * 
         * @param event The event. 
         */
        private void closeView(MouseEvent event) {
            int index = modules.indexOf(getModule(event.getPoint()));

            rectangle = getRectangle(event.getPoint());
            image = images[index];

            Animator springEffect = AnimationUtils.createSpringEffectAnimator(this);

            springEffect.addTarget(new TimingTargetAdapter(){
                @Override
                public void end() {
                    Managers.getManager(IViewManager.class).displaySplashScreen();
                }
            });

            Animator fadeOut = AnimationUtils.createFadeOutAnimator(this);

            TimingTrigger.addTrigger(springEffect, fadeOut, TimingTriggerEvent.STOP);

            springEffect.start();
        }

        /**
         * Launch the next phase. 
         * 
         * @param event The event. 
         */
        private void launchNextPhase(MouseEvent event) {
            Managers.getManager(IModuleManager.class).setPrimaryModule(getModule(event.getPoint()));

            if (Managers.getCore().getLifeCycleManager().isSecondPhaseDone()) {
                Managers.getManager(IModuleManager.class).launchPrimaryModule();
            } else {
                Managers.getCore().getLifeCycleManager().launchNextPhase();
            }
        }

        /**
         * Return the module rectangles at the specified point. 
         * 
         * @param point The point to get the rectangle for. 
         * 
         * @return the module rectangle or null if the point is not in a module rectangle. 
         */
        private Rectangle getRectangle(Point point){
            for (Map.Entry<ModuleContainer, Rectangle> entry : moduleRectangles.entrySet()) {
                if (entry.getValue().contains(point)) {
                    return entry.getValue();
                }
            }

            return null;
        }
    }

    /**
     * The mouse motion controller of the view. 
     * 
     * @author Baptiste Wicht
     */
    private final class MouseMotionController extends MouseMotionAdapter {
        @Override
        public void mouseMoved(MouseEvent event) {
            updateCursor(event);
            updateGrowEffect(event);
        }

        /**
         * Update the grow effect of the view. 
         * 
         * @param event The mouse event. 
         */
        private void updateGrowEffect(MouseEvent event) {
            if (eventInMouseShape(event)) {
                int index = modules.indexOf(getModule(event.getPoint()));

                if(grow != index){
           
                    grow = index;
                    repaint();
                }
            }
        }

        /**
         * Update the cursor. 
         * 
         * @param event The mouse event. 
         */
        private void updateCursor(MouseEvent event) {
            if (eventInMouseShape(event)) {
                setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
            } else {
                setCursor(Cursor.getDefaultCursor());
            }
        }
    }
}
