package org.jtheque.core.spring.aspect;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.ProceedingJoinPoint;
import org.apache.log4j.Logger;

/**
 * An AspectJ aspect to log the module plug, prePlug and unplug methods. 
 * 
 * @author Baptiste Wicht
 */
@Aspect
public final class ModuleLoggingAspect {
    /**
     * Log the plug() method of the modules. 
     * 
     * @param joinPoint The joinpoint of the advising method. 
     * 
     * @return null
     * 
     * @throws Throwable Throw if the plug method throws an exception. 
     */
    @Around("execution (* * ..*Module.plug())")
    public Object logPlug(ProceedingJoinPoint joinPoint) throws Throwable {
        Logger.getLogger(getClass()).trace(joinPoint.getSourceLocation().getWithinType().getSimpleName() + " plug started");
        
        Object retVal = joinPoint.proceed();
        
        Logger.getLogger(getClass()).trace(joinPoint.getSourceLocation().getWithinType().getSimpleName() + " plug finished");
        
        return retVal;
    }
    
    /**
     * Log the prePlug() method of the modules. 
     * 
     * @param joinPoint The joinpoint of the advising method. 
     * 
     * @return null
     * 
     * @throws Throwable Throw if the prePlug method throws an exception. 
     */
    @Around("execution (* * ..*Module.prePlug())")
    public Object logPrePlug(ProceedingJoinPoint joinPoint) throws Throwable {
        Logger.getLogger(getClass()).trace(joinPoint.getSourceLocation().getWithinType().getSimpleName() + " pre-plug started");
        
        Object retVal = joinPoint.proceed();
        
        Logger.getLogger(getClass()).trace(joinPoint.getSourceLocation().getWithinType().getSimpleName() + " pre-plug finished");
        
        return retVal;
    }
    
    /**
     * Log the unplug() method of the modules. 
     * 
     * @param joinPoint The joinpoint of the advising method. 
     * 
     * @return null
     * 
     * @throws Throwable Throw if the unplug method throws an exception. 
     */
    @Around("execution (* * ..*Module.unplug())")
    public Object logUnPlug(ProceedingJoinPoint joinPoint) throws Throwable {
        Logger.getLogger(getClass()).trace(joinPoint.getSourceLocation().getWithinType().getSimpleName() + " unplug started");
        
        Object retVal = joinPoint.proceed();
        
        Logger.getLogger(getClass()).trace(joinPoint.getSourceLocation().getWithinType().getSimpleName() + " unplug finished");
        
        return retVal;
    }
}